## Copyright (C) 2018 jdonoghue
## 
## This program is free software: you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program.  If not, see
## <https://www.gnu.org/licenses/>.

## -*- texinfo -*- 
## @deftypefn {} {@var{voltage} =} readVoltage (@var{ar}, @var{pin})
## Read analog voltage of  @var{pin}.
##
## @var{ar} - connected arduino.
## @var{pin} - pin name or number to query for voltage
## @var{voltage} - scaled pin value as a voltage
##
## Exmaple usage:
## @example
## @code
## ar = arduino ();
## readVoltage(ar, "A4");
## @end code
## ans = 
##     1.401
## @end example
## @seealso{arduino, readAnalogPin}
## @end deftypefn

function voltage = readVoltage (ar, pin)
  if nargin != 2
    error ("@arduino.readVoltage: expected pin name and value");
  endif
  if !ischar(pin)
    error ("@arduino.readVoltage: expected pin name as string");
  endif
%{ 
  pininfo = obj.get_pin(pin);

  # first use ?
  if strcmp(pininfo.mode, "unset")
    configurePin(obj, pin, "analoginput")
  else
    [pinstate, pinmode] = pinStateMode(pininfo.mode);
    if !strcmp(pinmode, "analog")
      error ("readVoltage: pin is in incompatable mode");
    endif
  endif
 
  datain = uint8([pininfo.id]);
  
  [dataout, status] = __sendCommand__ (obj, ARDUINO_ANALOG, datain);
 
  if status != 0
    error ("readVoltage: failed to set pin state err=%d", status);
  endif 
  
  retval = (dataout(1)*256 + dataout(2))/1000.0;
%}
  #voltage = readAnalogPin(ar,pin)/1000.0;
  voltage = readAnalogPin(ar,pin) * (5.0 / 1023.0);
endfunction
