/*
 * Octave arduino interface
 * Copyright (C) 2018 jdonoghue
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/
#include "settings.h"

#define ARDUINO_SOH         0xA5

#define ARDUINO_RESET       0
#define ARDUINO_INIT        1
#define ARDUINO_CONFIGPIN   2
#define ARDUINO_DIGITAL     3
#define ARDUINO_ANALOG      4
#define ARDUINO_PWM         5

#define ARDUINO_SPI         10
#define ARDUINO_CONFIGSPI   11

#define ARDUINO_SCANI2C     20
#define ARDUINO_CONFIGI2C   21
#define ARDUINO_WRITEI2C    22
#define ARDUINO_READI2C     23
#define ARDUINO_WRITEI2CREG 24
#define ARDUINO_READI2CREG  25

#define ARDION_SERVO        30
#define ARDUINO_CONFIG_SERVO 31

#define ARDUINO_ERROR       255

// TODO: how know what board we are ???
#define BOARD_ID  0
#define BOARD_VOLTAGE 5

#ifdef USE_SPI
#include <SPI.h>
uint8_t spi_enabled = false;
uint8_t spi_bitorder = 0;
uint8_t spi_mode = 0;
#endif

#ifdef USE_I2C
#include <Wire.h>
uint8_t i2c_enabled = false;
uint8_t i2c_address = 0;
#endif

uint8_t map_config_mode[] = {
  INPUT, // unset
  INPUT, // analoginput
  INPUT, // dig in
  OUTPUT, // dig out
  INPUT_PULLUP, // pullup
  OUTPUT, // i2c TODO: ??
  OUTPUT, // pwm
  OUTPUT, // servo
  OUTPUT, // spi TODO ?
};
   
uint8_t pinconfig[NUM_DIGITAL_PINS];
  
uint8_t s_state = 0;
uint8_t s_cmd = 0;
uint8_t s_ext = 0;
uint8_t s_sz = 0;
uint8_t s_data[256];
uint8_t s_datapos = 0;

void replyCommand (uint8_t *data, uint8_t len)
{
  Serial.write((uint8_t)ARDUINO_SOH);
  Serial.write(s_ext);
  Serial.write(s_cmd);
  Serial.write(len);
  if(len) {
    Serial.write(data, len);
  }
  Serial.flush();
}
  
int readCommand() {
    // SOH EXT CMD LEN [lendatabytes] 
    int ch;
    
    ch = Serial.read();

    //Serial,write('A');
    //Serial.write(ch);
    
    switch (s_state) {
      case 0: // SOH
        if(ch == ARDUINO_SOH)
          s_state = 1;
        break;
      case 1: // EXT
        s_ext = ch;
        s_state = 2;
        break;      
      case 2: // CMD
        s_cmd = ch;
        s_state = 3;
        break;
      case 3: // SZ
        s_sz = ch;
        s_datapos = 0;
        if(s_sz > 0)
          s_state = 4;
        else
          s_state = 5;
        break;
      case 4: // DATA
        if(s_datapos < sizeof(s_data))
          s_data[s_datapos] = ch;
        s_datapos ++;  
        if(s_datapos == s_sz)
          s_state = 5;
        break;
        
      default:
        s_state = 0;
        break;  
    }   
   
    if(s_state == 5) {
       s_state = 0;
       return s_cmd;
    } 
    return -1;
}

void processCommand() {
    int val;
    
    switch (s_cmd) {
      case ARDUINO_RESET:
        replyCommand(NULL, 0);
        break;
        
      case ARDUINO_INIT:
        s_data[0] =  SIGNATURE_0;
        s_data[1] =  SIGNATURE_1;
        s_data[2] =  SIGNATURE_2;
        s_data[3] = BOARD_ID;
        s_data[4] = BOARD_VOLTAGE;
        replyCommand(s_data, 5);
        break;  
#ifdef USE_I2C
      case ARDUINO_WRITEI2C:
      case ARDUINO_WRITEI2CREG:
       {
        if(s_sz < 2 || s_sz > 32)
        {
          // address
          // data
          s_cmd = ARDUINO_ERROR;
          replyCommand(NULL, 0);
        }
        else {
          Wire.beginTransmission(s_data[0]); // should be i2c_address
          byte c;
          for(c=1;c<s_sz;c++)
            Wire.write(s_data[c]);
          Wire.endTransmission();

          s_data[1] = 1;
          replyCommand(s_data, 2);
        }
        break;
       }
      case ARDUINO_READI2C:
       {
        if(s_sz != 2)
        {
          // address
          // numbytes
          
          s_cmd = ARDUINO_ERROR;
          replyCommand(NULL, 0);
        }
        else {
          Wire.requestFrom(s_data[0], s_data[1]);
          byte c = 0;
          byte l = s_data[1];
          s_sz = 1;
          for(c=0;c<=l;c++)
          {
            if(Wire.available()) {
              s_data[s_sz] = Wire.read();
              s_sz ++;
            }
          }
          replyCommand(s_data, s_sz);
        }
        break;
       }
      case ARDUINO_READI2CREG:
       {
        if(s_sz != 3)
        {
          // address
          // reg
          // numbytes
          
          s_cmd = ARDUINO_ERROR;
          replyCommand(NULL, 0);
        }
        else {
          Wire.beginTransmission(s_data[0]);
          Wire.write(s_data[1]);
          Wire.endTransmission(false);
          Wire.requestFrom(s_data[0], s_data[2]);
          byte c = 0;
          byte l = s_data[2];
          s_sz = 2;
          for(c=0;c<=l;c++)
          {
            if(Wire.available()) {
              s_data[s_sz] = Wire.read();
              s_sz ++;
            }
          }
          replyCommand(s_data, s_sz);
        }
        break;
       }

      case ARDUINO_SCANI2C:
       {
        if(s_sz != 2)
        {
          s_cmd = ARDUINO_ERROR;
          replyCommand(NULL, 0);
        }
        else {
          byte error;
          // bus 0
          // address
          if(!i2c_enabled)
            Wire.begin();

          Wire.beginTransmission(s_data[1]);
          error = Wire.endTransmission();

          if(error == 0)
            s_data[2] = 1;
          else
	    s_data[2] = 0;

          if(!i2c_enabled)
            Wire.end();

          replyCommand(s_data, 3);
        }

        break;
       }
      case ARDUINO_CONFIGI2C:
       {
         if(s_sz == 2 || s_sz == 3) {
           // i2c id   0
           // enable   1

           // enable
           if(s_data[1] == 1) {
             //SPI.begin();
             i2c_enabled = 1;
             i2c_address = 0;
             if (s_sz == 3 && s_data[2] != 0) {
               i2c_address = s_data[2];
               Wire.begin(i2c_address);
             }
             else
               Wire.begin();
           } else {
             Wire.end();
             i2c_enabled = 0;
           }
          replyCommand(s_data, s_sz);
         }
         else if(s_sz == 1) {
           // spi id
           // enable
           // address
          s_data[1] = i2c_enabled;
          s_data[2] = i2c_address;
          replyCommand(s_data, 3);
         }
         else {
          s_cmd = ARDUINO_ERROR;
          replyCommand(NULL, 0);
         }
         break;
       }
#endif

#ifdef USE_SPI        
      case ARDUINO_CONFIGSPI:
       {
         if(s_sz == 4) {
           // spi id   0
           // enable   1
           // mode ?   2
           // byte order 3
           // TODO: bit rate

           if(s_data[1] == 1) {
             //SPI.begin();
             spi_mode = s_data[2];
             spi_bitorder = s_data[3];
             spi_enabled = 1;
             if(spi_bitorder == 0) spi_bitorder = MSBFIRST;
             else spi_bitorder = LSBFIRST;

             if(spi_mode == 0) spi_mode = SPI_MODE0;
             else if(spi_mode == 1) spi_mode = SPI_MODE1;
             if(spi_mode == 2) spi_mode = SPI_MODE2;
             if(spi_mode == 3) spi_mode = SPI_MODE3;
             SPI.begin();
             SPI.beginTransaction(SPISettings(14000000, spi_bitorder, spi_mode));
           } else {
             SPI.endTransaction();
             SPI.end();
             spi_enabled = 0;
           }
          replyCommand(s_data, 2);
         }
         else if(s_sz == 2) {
           if(s_data[1] == 1) {
             spi_enabled = 1;
             SPI.begin();
             SPI.beginTransaction(SPISettings(14000000, spi_bitorder, spi_mode));
           } else {
             SPI.endTransaction();
             SPI.end();
             spi_enabled = 0;
           }
          replyCommand(s_data, 2);
 
         }
         else if(s_sz == 1) {
           // spi id
           // enable
           // mode ?
           // byte order
          s_data[1] = spi_enabled;
          s_data[2] = spi_mode;
          s_data[3] = spi_bitorder;
          replyCommand(s_data, 4);
         }
         else {
          s_cmd = ARDUINO_ERROR;
          replyCommand(NULL, 0);
         }
         break;
       }
#endif
      case ARDUINO_CONFIGPIN:
        if (s_sz == 1 && s_data[0] < NUM_DIGITAL_PINS) {
          s_data[1] = pinconfig[s_data[0]]; // TODO: get mode somehow ????
          replyCommand(s_data, 2);      
        } else if (s_sz == 2 && s_data[0] < NUM_DIGITAL_PINS && s_data[1] >= 0 && s_data[1] < sizeof(map_config_mode)) {
          int mode = map_config_mode[s_data[1]];
          pinconfig[s_data[0]] = s_data[1];
          pinMode(s_data[0], mode);
          replyCommand(NULL, 0);
        } else {
          s_cmd = ARDUINO_ERROR;
          replyCommand(NULL,0);
        }
        break; 
        
      case ARDUINO_DIGITAL:
      
        if(s_sz == 1) {
          val = digitalRead(s_data[0]);
          if(val == HIGH)
            s_data[1] = 1;
          else
            s_data[1] = 0;
          replyCommand(s_data, 2);
        } else if(s_sz == 2) {
          digitalWrite(s_data[0], s_data[1]); 
          replyCommand(NULL, 0);
        } else {
          s_cmd = ARDUINO_ERROR;
          replyCommand(NULL,0);
        }
        break;  

      case ARDUINO_ANALOG:
      
        if(s_sz == 1) {
          val = analogRead(s_data[0]);
          s_data[1] = (val>>8)&0xff;
          s_data[2] = (val)&0xff;
          replyCommand(s_data, 3);
        } else {
          s_cmd = ARDUINO_ERROR;
          replyCommand(NULL, 0);
        }
        break;  

      case ARDUINO_PWM:
      
        if(s_sz == 2) {
          analogWrite(s_data[0], s_data[1]);
          replyCommand(NULL, 0);
        } else {
          s_cmd = ARDUINO_ERROR;
          replyCommand(NULL, 0);
        }
        break;  

#ifdef USE_SPI        
      case ARDUINO_SPI:
      
        if(s_sz == 2) {
          s_data[1] = SPI.transfer(s_data[1]);
          replyCommand(s_data, 2);
        } else {
          s_cmd = ARDUINO_ERROR;
          replyCommand(NULL, 0);
        }
        break;  
#endif               
      default:
        s_cmd = ARDUINO_ERROR;
        replyCommand(NULL, 0);
        break;
    }
}

void setup() {
  // set pins as not set
  for(int i = 0;i<NUM_DIGITAL_PINS;i++) {
    pinconfig[i] = 0xff;
  }
  Serial.begin(9600); //set up serial library baud rate to 9600
}

void loop() {
  if(Serial.available()) {
     if(readCommand() != -1) {
       processCommand();
     }
  }
}
